library(foreign)

## a function to compute rates from weighted data
compute.prob <- function(weights, num, den) {
  return (sum(weights[num], na.rm = TRUE)/sum(weights[den], na.rm = TRUE))
}

## load the dataset
incidents <- read.spss("NCVS - Incidents - 2015.sav", add.undeclared.levels="no", to.data.frame = TRUE)

## subset data to keep only entries about violent crimes
## (note: I have excluded verbal threats of violence)
violent.crimes <- incidents[as.integer(incidents$V4529) <= 17, ]

## calculate the proportion of violent crimes committed by a single non-hispanic male white offender
white.prop <- compute.prob(violent.crimes$V4527, violent.crimes$V4236 == "Male" & violent.crimes$V4246B == "Yes" &
                           violent.crimes$V4246C == "No" & violent.crimes$V4246D == "No" &
                           violent.crimes$V4246E == "No" & violent.crimes$V4246F == "No" &
                           violent.crimes$V4237A == "No", violent.crimes$V4246A == "At least 1 entry" &
                           !is.na(violent.crimes$V4237A) & violent.crimes$V4237A != "Don't know" &
                           violent.crimes$V4237A != "Residue")

## calculate the proportion of violent crimes committed by a single non-hispanic male black offender
black.prop <- compute.prob(violent.crimes$V4527, violent.crimes$V4236 == "Male" & violent.crimes$V4246B == "No" &
                           violent.crimes$V4246C == "Yes" & violent.crimes$V4246D == "No" &
                           violent.crimes$V4246E == "No" & violent.crimes$V4246F == "No" &
                           violent.crimes$V4237A == "No", violent.crimes$V4246A == "At least 1 entry" &
                           !is.na(violent.crimes$V4237A) & violent.crimes$V4237A != "Don't know" &
                           violent.crimes$V4237A != "Residue")

## calculate the proportion of violent crimes committed by a single hispanic male offender
hisp.prop <- compute.prob(violent.crimes$V4527, violent.crimes$V4236 == "Male" & violent.crimes$V4237A == "Yes", !is.na(violent.crimes$V4237A) &
                          violent.crimes$V4237A != "Don't know" & violent.crimes$V4237A != "Residue")

## calculate black/white ratio of offending rates for violent crimes based on population 15 or above
## (note: for the population age 15 or above in 2015 in various groups, I used the figures given in
## the documentation of the NAMCS for that year, which are decomposed by age groups, race and ethnicity)
bw.ratio <- (black.prop/13749722)/(white.prop/79783015)

## calculate hispanic/white ratio of offending rates for violent crimes based on population 15 or above
## (note: for the population age 15 or above in 2015 in various groups, I used the figures given in
## the documentation of the NAMCS for that year, which are decomposed by age groups, race and ethnicity)
hw.ratio <- (hisp.prop/20257393)/(white.prop/79783015)